package de.unikassel.ir.vsr;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import junit.framework.TestCase;

public class PhraseSearchIndexTest extends TestCase {
	private Corpus _corpus;

	private PhraseSearchIndex _psix;

	/**
	 * Corpus und Index bauen
	 */
	public void setUp() {
		_corpus = new CorpusImpl();

		File dir = new File("texte");
		for (File file : dir.listFiles()) {
			try {
				if (!file.isDirectory()) { // CVS-Zeug überspringen ;-)
					FileInputStream stream = new FileInputStream(file);
					Document doc = new DocumentImpl(file.getName());
					doc.read(stream);
					stream.close();
					_corpus.addDocument(doc);
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		_psix = new PhraseSearchIndex(_corpus);
	}

	/**
	 * Testen, ob eine Phrase, die nicht gefunden wird,
	 * eine leere Ergebnismenge liefert.
	 */
	public void testIndexEmpty() {
		String phrase = "wer das findet liegt falsch";
		Map<Document, Vector<Integer>> result = _psix.searchPhrase(phrase);
		Map<String, int[]> expected = new HashMap<String, int[]>();
		checkResult(result, expected);
	}
	
	/**
	 * Test: Phrase 1
	 */
	public void testIndexSugar() {
		String phrase = "tastes like sugar";
		Map<Document, Vector<Integer>> result = _psix.searchPhrase(phrase);
		Map<String, int[]> expected = new HashMap<String, int[]>();
		expected.put("Reut_3589.txt", new int[] { 60 });
		expected.put("Reut_3577.txt", new int[] { 61 });
		checkResult(result, expected);
	}
	
	/**
	 * Test: Phrase 2
	 */
	public void testIndexCommittee() {
		String phrase = "executive committee";
		Map<Document, Vector<Integer>> result = _psix.searchPhrase(phrase);
		Map<String, int[]> expected = new HashMap<String, int[]>();
		expected.put("Reut_10122.txt", new int[] { 245 });
		expected.put("Reut_10760.txt", new int[] { 712 });
		expected.put("Reut_7311.txt", new int[] { 28, 90 });
		expected.put("Reut_7512.txt", new int[] {296 });
		checkResult(result, expected);
	}

	/**
	 * Resultatmenge mit etwartetem Resultat (liegt in etwas anderer Form vor)
	 * vergleichen.
	 * 
	 * @param result das erhaltene Resultat
	 * @param expected das erwartete Resultat
	 */
	public void checkResult(Map<Document, Vector<Integer>> result,
			Map<String, int[]> expected) {
		// Richtige Anzahl Dokumente?
		assertEquals(expected.size(), result.size());

		// für alle Dokumente: Fundstellen prüfen
		for (Document doc : result.keySet()) {
			assertTrue(expected.containsKey(doc.getId()));

			Vector<Integer> posResult = result.get(doc);
			int[] expResult = expected.get(doc.getId());

			assertEquals(posResult.size(), expResult.length);

			for (int pos : expResult) {
				assertTrue(posResult.contains(pos));
			}
		}

	}
}